package io.hackle.hackle;

import androidx.annotation.NonNull;

import java.util.logging.Level;
import java.util.logging.Logger;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;


/**
 * HacklePlugin
 */
public class HacklePlugin implements FlutterPlugin, MethodCallHandler {
    private MethodChannel channel;
    private MethodParser methodParser;
    private final Logger logger = Logger.getLogger("HacklePlugin");

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        channel = new MethodChannel(flutterPluginBinding.getBinaryMessenger(), "hackle");
        channel.setMethodCallHandler(this);
        this.methodParser = new MethodParser(flutterPluginBinding.getApplicationContext());
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        if (methodParser != null) {
            methodParser.onMethodCall(call.method, call.arguments, result);
        } else {
            result.error("METHOD_PARSER_NOT_INITIALIZED", null, null);
            logger.log(Level.WARNING, "Method parser failed to initialize");
        }
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
        methodParser = null;
    }
}
