package io.hackle.hackle;

import androidx.annotation.NonNull;

import java.util.HashMap;

import io.hackle.android.HackleApp;
import io.hackle.android.HackleConfig;
import io.hackle.sdk.common.Event;
import io.hackle.sdk.common.ParameterConfig;
import io.hackle.sdk.common.User;
import io.hackle.sdk.common.decision.Decision;
import io.hackle.sdk.common.decision.FeatureFlagDecision;

public class ModelConverter {
    User convertHashMapToUser(@NonNull HashMap<String, Object> map, @NonNull String defaultId) {
        final String id = (String) map.get("id");
        final String deviceId = (String) map.get("deviceId");
        final String userId = (String) map.get("userId");

        final User.Builder builder = User.builder(id != null ? id : defaultId)
                .deviceId(deviceId != null ? deviceId : defaultId);

        if (userId != null) {
            builder.userId(userId);
        }

        final HashMap<String, String> identifiers = (HashMap<String, String>) map.get("identifiers");
        for (HashMap.Entry<String, String> entry : identifiers.entrySet()) {
            builder.identifier(entry.getKey(), entry.getValue());
        }

        final HashMap<String, Object> properties = (HashMap<String, Object>) map.get("properties");

        for (HashMap.Entry<String, Object> entry : properties.entrySet()) {
            final String propertyKey = entry.getKey();
            final Object value = entry.getValue();

            if (value instanceof String) {
                builder.property(propertyKey, (String) value);
            }

            if (value instanceof Integer) {
                builder.property(propertyKey, (Integer) value);
            }

            if (value instanceof Long) {
                builder.property(propertyKey, (Long) value);
            }

            if (value instanceof Double) {
                builder.property(propertyKey, (Double) value);
            }

            if (value instanceof Boolean) {
                builder.property(propertyKey, (Boolean) value);
            }
        }

        return builder.build();
    }

    HackleConfig convertHashMapToHackleConfig(@NonNull HashMap<String, Object> map) {
        return HackleConfig.builder()
                .add("$wrapper_name", (String) map.get("$wrapperName"))
                .add("$wrapper_version", (String) map.get("$wrapperVersion"))
                .exposureEventDedupIntervalMillis((Integer) map.get("exposureEventDedupIntervalMillis"))
                .build();
    }

    Event convertHashMapToEvent(@NonNull HashMap<String, Object> map) {
        final String key = (String) map.get("key");
        final Double value = (Double) map.get("value");

        final Event.Builder builder = Event.builder(key);

        if (value != null) {
            builder.value(value);
        }

        final HashMap<String, Object> properties = (HashMap<String, Object>) map.get("properties");
        for (HashMap.Entry<String, Object> entry : properties.entrySet()) {
            final String propertyKey = entry.getKey();
            final Object propertyValue = entry.getValue();

            if (propertyValue instanceof String) {
                builder.property(propertyKey, (String) propertyValue);
            }

            if (propertyValue instanceof Integer) {
                builder.property(propertyKey, (Integer) propertyValue);
            }

            if (propertyValue instanceof Long) {
                builder.property(propertyKey, (Long) propertyValue);
            }

            if (propertyValue instanceof Double) {
                builder.property(propertyKey, (Double) propertyValue);
            }

            if (propertyValue instanceof Boolean) {
                builder.property(propertyKey, (Boolean) propertyValue);
            }
        }

        return builder.build();
    }


    HashMap<String, Object> convertParameterConfigToHashMap(@NonNull final ParameterConfig config) {
        return new HashMap<String, Object>() {{
            put("parameters", new HashMap<>(config.getParameters()));
        }};
    }

    HashMap<String, Object> convertDecisionToHashMap(@NonNull final Decision decision) {
        return new HashMap<String, Object>() {{
            put("variation", decision.getVariation().toString());
            put("reason", decision.getReason().toString());
            put("config", convertParameterConfigToHashMap(decision.getConfig()));
        }};
    }

    HashMap<String, Object> convertFeatureFlagDecisionToHashMap(@NonNull final FeatureFlagDecision featureFlagDecision) {
        return new HashMap<String, Object>() {{
            put("isOn", featureFlagDecision.isOn());
            put("reason", featureFlagDecision.getReason().toString());
            put("config", convertParameterConfigToHashMap(featureFlagDecision.getConfig()));
        }};
    }
}

